@@ -2,7 +2,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.QueryUtils
--- Copyright   :  (c) Sven Panne 2002-2006
+-- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -22,19 +22,31 @@ module Graphics.Rendering.OpenGL.GL.QueryUtils (
    getEnum1,
    getSizei1,
    getFloat1, getFloat2, getFloat3, getFloat4, getFloatv,
-   getDouble1, getDouble2, getDouble4, getDoublev
+   getDouble1, getDouble2, getDouble4, getDoublev,
+   maybeNullPtr,
+   AttribLocation(..), GetVertexAttribPName(..),
+   getVertexAttribInteger1, getVertexAttribEnum1, getVertexAttribBoolean1,
+   getVertexAttribFloat4, getVertexAttribIInteger4, getVertexAttribIuInteger4, 
+   GetVertexAttribPointerPName(..), getVertexAttribPointer
 ) where
 
 import Foreign.Marshal.Alloc ( alloca )
 import Foreign.Marshal.Array ( allocaArray )
-import Foreign.Ptr ( Ptr )
+import Foreign.Ptr ( Ptr, nullPtr )
+import Foreign.Storable ( Storable(peek) )
 import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLenum, GLint, GLsizei, GLfloat, GLdouble )
+   GLboolean, GLenum, GLint, GLsizei, GLuint, GLfloat, GLdouble )
+import Graphics.Rendering.OpenGL.GL.Extensions (
+   FunPtr, unsafePerformIO, Invoker, getProcAddress )
 import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1, peek2, peek3, peek4 )
 import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidEnum )
 
 --------------------------------------------------------------------------------
 
+#include "HsOpenGLExt.h"
+
+--------------------------------------------------------------------------------
+
 data GetPName =
      GetCurrentColor
    | GetCurrentIndex
@@ -356,6 +368,7 @@ data GetPName =
    | GetMaxTextureUnits
    | GetTextureCubeMap
    | GetMaxCubeMapTextureSize
+   | GetMaxRectangleTextureSize
    | GetNumCompressedTextureFormats
    | GetCompressedTextureFormats
    | GetMaxVertexUnits
@@ -413,8 +426,10 @@ data GetPName =
    | GetPixelPackBufferBinding
    | GetPixelUnpackBufferBinding
    | GetDrawBufferN GLsizei
+   | GetRGBASignedComponents
+   | GetCopyReadBuffer
+   | GetCopyWriteBuffer
    -- GetWeightArrayBufferBinding
-   -- GetVertexAttribArrayBufferBinding
 
 marshalGetPName :: GetPName -> Maybe GLenum
 marshalGetPName x = case x of
@@ -738,6 +753,7 @@ marshalGetPName x = case x of
    GetMaxTextureUnits -> Just 0x84e2
    GetTextureCubeMap -> Just 0x8513
    GetMaxCubeMapTextureSize -> Just 0x851c
+   GetMaxRectangleTextureSize -> Just 0x84F8
    GetNumCompressedTextureFormats -> Just 0x86a2
    GetCompressedTextureFormats -> Just 0x86a3
    GetMaxVertexUnits -> Just 0x86a4
@@ -795,8 +811,10 @@ marshalGetPName x = case x of
    GetPixelPackBufferBinding -> Just 0x88ed
    GetPixelUnpackBufferBinding -> Just 0x88EF
    GetDrawBufferN i -> drawBufferIndexToEnum i
+   GetRGBASignedComponents -> Just 0x8C3C
+   GetCopyReadBuffer -> Just 0x8F36
+   GetCopyWriteBuffer -> Just 0x8F37
    -- GetWeightArrayBufferBinding -> Just 0x889e
-   -- GetVertexAttribArrayBufferBinding -> Just 0x889f
 
 --------------------------------------------------------------------------------
 
@@ -949,3 +967,90 @@ getDoublev = maybe (const recordInvalidEnum) glGetDoublev . marshalGetPName
 
 foreign import CALLCONV unsafe "glGetDoublev" glGetDoublev ::
    GLenum -> Ptr GLdouble -> IO ()
+
+--------------------------------------------------------------------------------
+
+maybeNullPtr :: b -> (Ptr a -> b) -> Ptr a -> b
+maybeNullPtr n f ptr | ptr == nullPtr = n
+                     | otherwise      = f ptr
+--------------------------------------------------------------------------------
+
+newtype AttribLocation = AttribLocation GLuint
+   deriving ( Eq, Ord, Show )
+
+--------------------------------------------------------------------------------
+
+data GetVertexAttribPName =
+     GetVertexAttribArrayEnabled
+   | GetVertexAttribArraySize
+   | GetVertexAttribArrayStride
+   | GetVertexAttribArrayType
+   | GetVertexAttribArrayNormalized
+   | GetCurrentVertexAttrib
+   | GetVertexAttribArrayBufferBinding
+   | GetVertexAttribArrayInteger
+
+marshalGetVertexAttribPName :: GetVertexAttribPName -> GLenum
+marshalGetVertexAttribPName x = case x of
+   GetVertexAttribArrayEnabled -> 0x8622
+   GetVertexAttribArraySize -> 0x8623
+   GetVertexAttribArrayStride -> 0x8624
+   GetVertexAttribArrayType -> 0x8625
+   GetVertexAttribArrayNormalized -> 0x886A
+   GetCurrentVertexAttrib -> 0x8626
+   GetVertexAttribArrayBufferBinding -> 0x889F
+   GetVertexAttribArrayInteger -> 0x88FD
+
+--------------------------------------------------------------------------------
+
+getVertexAttribInteger1 :: (GLint -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribInteger1 f location n = alloca $ \buf -> do
+   glGetVertexAttribivARB location (marshalGetVertexAttribPName n) buf
+   peek1 f buf
+
+getVertexAttribEnum1 :: (GLenum -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribEnum1 f = getVertexAttribInteger1 (f . fromIntegral)
+
+getVertexAttribBoolean1 :: (GLboolean -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribBoolean1 f = getVertexAttribInteger1 (f . fromIntegral)
+
+EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glGetVertexAttribivARB,AttribLocation -> GLenum -> Ptr GLint -> IO ())
+
+getVertexAttribFloat4 :: (GLfloat -> GLfloat -> GLfloat -> GLfloat -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribFloat4 f location n = alloca $ \buf -> do
+   glGetVertexAttribfvARB location (marshalGetVertexAttribPName n) buf
+   peek4 f buf
+
+EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glGetVertexAttribfvARB,AttribLocation -> GLenum -> Ptr GLfloat -> IO ())
+
+getVertexAttribIInteger4 :: (GLint -> GLint -> GLint -> GLint -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribIInteger4 f location n = alloca $ \buf -> do
+   glGetVertexAttribIivARB location (marshalGetVertexAttribPName n) buf
+   peek4 f buf
+
+EXTENSION_ENTRY("GL_EXT_gpu_shader4 or OpenGL 3.0",glGetVertexAttribIivARB,AttribLocation -> GLenum -> Ptr GLint -> IO ())
+
+getVertexAttribIuInteger4 :: (GLuint -> GLuint -> GLuint -> GLuint -> b) -> AttribLocation -> GetVertexAttribPName -> IO b
+getVertexAttribIuInteger4 f location n = alloca $ \buf -> do
+   glGetVertexAttribIuivARB location (marshalGetVertexAttribPName n) buf
+   peek4 f buf
+
+EXTENSION_ENTRY("GL_EXT_gpu_shader4 or OpenGL 3.0",glGetVertexAttribIuivARB,AttribLocation -> GLenum -> Ptr GLuint -> IO ())
+
+--------------------------------------------------------------------------------
+
+data GetVertexAttribPointerPName =
+   VertexAttribArrayPointer
+
+marshalGetVertexAttribPointerPName :: GetVertexAttribPointerPName -> GLenum
+marshalGetVertexAttribPointerPName x = case x of
+   VertexAttribArrayPointer -> 0x8645
+
+--------------------------------------------------------------------------------
+
+getVertexAttribPointer :: AttribLocation -> GetVertexAttribPointerPName -> IO (Ptr a)
+getVertexAttribPointer location n = alloca $ \buf -> do
+   glGetVertexAttribPointervARB location (marshalGetVertexAttribPointerPName n) buf
+   peek buf
+
+EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glGetVertexAttribPointervARB,AttribLocation -> GLenum -> Ptr (Ptr a) -> IO ())
